#pragma once
#ifndef tsqueue_hpp
#define tsqueue_hpp 1
#include <condition_variable> 
#include <iostream> 
#include <mutex> 
#include <deque> 

// Thread-safe queue 
template <typename T> 
class TSQueue { 
private: 
    // Underlying queue 
    std::deque<T> m_queue; 

    // mutex for thread synchronization 
    std::mutex m_mutex; 

    // Condition variable for signaling 
    std::condition_variable m_cond; 

public: 
    std::deque<T> queue(){
        return m_queue;
    }

    size_t size(){
        return m_queue.size();
    }

    // Pushes an element to the queue 
    void push(const T &item){ 

        // Acquire lock 
        std::unique_lock<std::mutex> lock(m_mutex); 

        // Add item 
        m_queue.push_back(item); 

        // Notify one thread that 
        // is waiting 
        m_cond.notify_one(); 
    } 

    // Pops an element off the queue 
    T pop(){ 

        // acquire lock 
        std::unique_lock<std::mutex> lock(m_mutex); 

        // wait until queue is not empty 
        /*m_cond.wait(lock, 
                    [this]() { return !m_queue.empty(); }); */

        // retrieve item 
        T &item = m_queue.front(); 
        m_queue.pop_front(); 

        // return item 
        return item; 
    } 
}; 
#endif